#include "state_data.h"
#include <xhtml_doc.h>
#include <algorithm>
#include <string>
#include <locale>
#include <iomanip>



namespace xport_samples
{
	template<typename T> void generate_state_square_area_report(T& doc);
	template<typename T> void create_header(T& doc);
	template<typename T> void create_report_body(T& doc, const std::vector<state_info>& siv);
	template<typename T> void create_footer(T& doc);
	template<typename T> void create_stylesheet(T& doc);
	template<typename T> void create_table_header(T& tbl, int rgn);
	template<typename T> void create_region_table(T& tbl, const std::vector<state_info>& siv, int rgn);
	std::string get_region_name(int region);
	template<typename T> std::string numeric_format(T num, int dec);

	bool region_area_comp(const state_info& lhs, const state_info& rhs) { return lhs.region() * 100 + lhs.area_rank() < rhs.region() * 100 + rhs.area_rank(); }

	struct region_comp
	{
		region_comp(int rgn) : region(rgn) {}
		bool operator() (const state_info& si) { return si.region() == region; }
		int region;
	};

}

template<typename T>
void xport_samples::generate_state_square_area_report(T& doc)
{
	// define some handy typedefs
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;
	typedef xhtml_markup<T::doc_type, char> markup;

	const std::string file_path = "states.txt";

	// load and sort data
	std::vector<state_info> siv;
	load_state_data(file_path, siv);
	std::sort(siv.begin(), siv.end(), &region_area_comp);

	std::vector<state_info>::const_iterator it = siv.begin();
	for (; it != siv.end(); ++it) {
		std::string test = it->name();
	}

	create_header(doc);
	create_report_body(doc, siv);
	create_footer(doc);
	create_stylesheet(doc);

}


template<typename T> 
void xport_samples::create_header(T& doc)
{
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;
	typedef xhtml_markup<T::doc_type, char> markup;

	element head_div(Xport::div, "id_heading");

	typename markup::iterator it = head_div.insert(element(h1));
	*it << "US State Area Report";

	it = head_div.insert(element(h4));
	*it << "Each state is listed within it's region, along with Regional subtotals";

	element lnk(a);
	lnk << "Wikipedia";
	lnk.attribute(href, "http://en.wikipedia.org/wiki/Wiki");

	*doc.body() << head_div;

	*doc.body() << (element(Xport::div, "id_aknowledgement") << "Statistics courtesy of " << lnk);
}

template<typename T> 
void xport_samples::create_report_body(T& doc, const std::vector<state_info>& siv)
{
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;
	typedef xhtml_markup<T::doc_type, char> markup;

	std::vector<state_info>::const_iterator it = siv.begin(), it_end = siv.end();
	int rgn = 1;
	for (int rgn = 1; rgn <= 4; ++it, ++rgn) {
		std::ostringstream sRgn;
		sRgn << "Region" << rgn;
		element tbl(table, sRgn.str());
		create_region_table(tbl, siv, rgn);
		doc.body()->insert(tbl);
	}
}

template<typename T> 
void xport_samples::create_region_table(T& tbl, const std::vector<state_info>& siv, int rgn)
{
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;
	typedef xhtml_markup<T::doc_type, char> markup;

	create_table_header(tbl, rgn);
	std::vector<state_info>::const_iterator it = std::find_if(siv.begin(), siv.end(), region_comp(rgn));
	int rows = 0;
	for (; it != siv.end() && it->region() == rgn; ++it, ++rows) {
		typename markup::iterator mit = tbl.insert(element(tr));
		*mit << (element(td) << it->name());
		*mit << (element(td) << it->abbr());
		*mit << (element(td) << numeric_format(it->area(), 0));
		std::ostringstream sArea_rank;
		sArea_rank << it->area_rank();
		*mit << (element(td) << sArea_rank.str());
	}

	// now go back and insert the image cell in the first row, with a rowspan of all rows
	typename markup::iterator mit = tbl.begin();
	++mit;
	mit = mit->insert(element(td, "", "img_cell"));
	std::ostringstream attr_val;
	attr_val << rows + 1;
	mit->attribute(rowspan, attr_val.str());
	
	// insert image in image cell
	mit = mit->insert(element(img));
	attr_val.str("");
	attr_val << "region_" << rgn << ".gif";
	mit->attribute(src, attr_val.str());

}



template<typename T> 
void xport_samples::create_table_header(T& tbl, int rgn)
{
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;
	typedef xhtml_markup<T::doc_type, char> markup;

	typename markup::iterator it = tbl.insert(element(tr));
	*it << (element(th) << get_region_name(rgn));

	it = tbl.insert(element(tr, "", "titles"));
	*it << (element(th) << "State");
	*it << (element(th) << "Abbr");
	*it << (element(th) << "Area (sq miles)");
	*it << (element(th) << "Area (Rank)");

}


template<typename T> 
void xport_samples::create_footer(T& doc)
{

}

template<typename T> 
void xport_samples::create_stylesheet(T& doc)
{
	using namespace Xport;
	typedef xhtml_element<T::doc_type, char> element;

	element css_link(link);
	css_link.attribute(rel, "stylesheet");
	css_link.attribute(type, "text/css");
	css_link.attribute(href, "us_state_sqare_area_report.css");
	*doc.head() << css_link;
}

std::string xport_samples::get_region_name(int region)
{
	return "";
}

template<typename T> 
std::string xport_samples::numeric_format(T num, int dec)
{
	std::ostringstream sNum;
	sNum << std::setprecision(dec) << num;
	std::string result(sNum.str());
	std::string::reverse_iterator rit = result.rbegin(), rit_end = result.rend();

	std::string result2;
	std::string::size_type dec_idx = result.find('.');
	if (dec_idx != std::string::npos) {
		rit += result.size() - dec_idx;
		result2 = result.substr(dec_idx);
		std::reverse(result2.begin(), result2.end());
	}

	int thous_idx = 0;
	for (; rit != rit_end; ++rit, ++thous_idx) {
		result2.push_back(*rit);
		if (thous_idx >= 3) {
			result2.push_back(',');
		}
	}

	std::reverse(result2.begin(), result2.end());

	return result2;
}


